/********************************************
 *                                          *
 *   SMPC SAMPLE PROGRAM Version 0.56a      *
 *                                          *
 *    file : smpcdem.c                      *
 *                                          *
 *  author : s.uchida sega cs h/w r&d       *
 *                                          *
 *    date : 94/11/09                       *
 *                                          *
 *	   rev : a.stern soa npe v.01           *
 *	  date : 11/30/94                       *
 *     rev : a.stern soa v695               *
 *    date : 06/12/95                       *
 *                                          *
 *    Peripheral Port Monitor PPMSPR using  *
 *    SMPC Intback Command (Control Mode)   *
 *    used to get peripheral and system data*
 *                                          *
 *	  One Intback command per Vblank        *
 *    At least two SMPC Interrupts per Vblank
 *    Second SMPC Interrupt will have upto  *
 *    32bytes of peripheral data            *
 *                                          *
 ********************************************/


#define _GLOBAL_SUBSTANCE_

#define Static static

#include "ssp.h"


Static Sint8 itos_work[32];
Static Sint8 hex[] = "0123456789ABCDEF";
Static Sint8 *timeout[] = {"  ","TO"};
Static Sint8 *week[] = {"SUN","MON","TUE","WED","THU","FRI","SAT"};
Static Sint8 guide[2][4][12] = {
	" C+START","        ",
	" TO MENU","        ",
	" ab:MOVE","        ",
	"A/C:EXEC","        "
};

Static Uint8  SmpcIntCount;


/*********************
		 MAIN PROGRAM
 *********************/

main()
{
	Uint16 i,j,k,l,m,n;
	Uint8 x,y,r,g,b;
	MenuEnable = OFF;
/*ADEBUG*/
	ProgramMode = PPMSPR;		/* peripheral port monitor */
 /*ProgramMode = MENU;*/

	mm.lmenux = 0;
	mm.lmenuy = 0;
	SetScreenMode();
	Init();

	SpCmdInit();
	SetVblank();		 /*setup routines to run on vblkIN vblkOUT and smpc int*/
	set_imask(0);
	SCL_SetFrameInterval(0);
	SCL_DisplayFrame();

	pokeb(0x2010007d,0);			/* IOSEL bit0=p1 bit1=p2 select control mode=0 */
	pokeb(0x2010007f,0);      /* EXLE  bit0=p1 bit1=p2 use bit6 as I/O port */



	while(1){						/* BACKGROUND TASKS */
		switch(ProgramMode){
			case MENU:
				MainMenu();
				break;
			case PPMSPR:
				if(!SMPCBUSY)
					PortMoni();
				break;
			case PPMRBG:
				RbgPortMoni();
				break;
			case OREGSMP:
				OregSampler();
				break;
			case PAINT:
				PaintTool();
				break;
			case CMDTEST:
				smpccommand();
				break;
			default:
				NotAvailable();
				break;
		}
	}
}



/******************************************
 *                                        *
 * PRINT OREGS
 *                                        *
 ******************************************/



void PrintOreg(void)
{
	Uint16 i;
	Uint8 j;

	for(i=0;i<128;i++)
		putstr((i % 8) * 3 + 40,i / 8 + 3 + TOPLINE,itosh2(oreg[i]),WHITE);

}


/******************************************
 *                                        *
 * PRINT STRING
 *                                        *
 ******************************************/



void putstr(Uint8 x,Uint8 y,Sint8 *string,Uint8 color)
{
	Uint32 *vram;

	vram = (Uint32 *)SCL_VDP2_VRAM_B0 + x + y * 0x40;

	while(*string){
			(*(Uint32 *)vram) = (Uint32)(0x00000000+(*string)*2);
		vram++;
		string++;
	}
}


/******************************************
 *                                        *
 * INTEGER TO STRING
 *                                        *
 ******************************************/


Sint8 *itos(Uint32 num,Uint8 v,Uint8 c)
{

	Sint8 i,*p;

	p = itos_work + c;

	*p-- = '\0';

	for(i = 0;i < c; i++){
		*p-- = hex[num % v];
		num /= v;
	}

	return itos_work;
}


/******************************************
 *                                        *
 * INTEGER TO STRING (NUM)
 *                                        *
 ******************************************/


Sint8 *itosh2(Uint32 num)
{

	*itos_work       = hex[(num & 0xf0)>>4];
	*(itos_work + 1) = hex[num & 0x0f];
	*(itos_work + 2) = '\0';

	return itos_work;
}



/*----------------------------------------*
 *                                        *
 * ISSUE INTBACK SMPC COMMAND             *
 *                                        *
 -----------------------------------------*/


void intback(void)
{
	Uint8 i;

	if(SMPCBUSY) return;
	contflag = 0x01;

	pokeb(0x20100001,0x01); /* IREG0 Status Acquisition Switch 1=all data */
	pokeb(0x20100003,0x0a); /* IREG1 Data Collection Mode 0000 1000 */
													/* bits76 p1=15 bytes mode bits54 p2=15 bytes mode*/
													/* bit3    Peripheral Data Returned */
													/* bit1    Acqusition Time Optimization OFF */
	pokeb(0x20100005,0xf0); /* IREG2 ALWAYS $F0 */
	pokeb(0x20100063,0x01); /* Status Flag set by SH-2, cleared by SMPC */
	pokeb(0x2010001f,0x10); /* Command Register INTBACK Command Code $10 */
}


/******************************************
 *                                        *
 * Vblank In ISR
 *                                        *
 ******************************************/


void UserVblankStart(void)
{
	Uint32 i;
	Uint8 j,k;

	per_byte = devptr - 31;


	if(SMPCBUSY) {							/* print while SMPC not done */
		putstr(1,BORDERLINE,timeout[VBinCount>>2 & 1],WHITE);
		return;
	}

	putstr(1,BORDERLINE,"  ",WHITE);
	SCL_VblankStart();

	getportstat();							/* Copy OREG[] into device[] */


	PrintOreg();


		switch(ProgramMode){
			case MENU:
 				SpDemo();
				rtc();
				intback();
				break;
			case PPMSPR:
				screenout();
				pad_action();
				rtc();
				intback();
				break;
			case PPMRBG:
				screenout();
				rbg_pad_action(MenuDev);
				SpDemo();
				rtc();
				intback();
				break;
			case OREGSMP:
				SpDemo();
				oregsmp_sub();
				rtc();
				intback();
				break;
			case PAINT:
				Paint_Main();
				rtc();
				intback();
				break;
			case CMDTEST:
				SpDemo();
				rtc();
				intback();
				break;
			default:
				SpDemo();
				break;
		}


	putstr(17,BORDERLINE,itosh2(MainLoopCount >> 16),WHITE);
	putstr(20,BORDERLINE,itosh2((VBinCount++) / 16),WHITE);
	putstr(23,BORDERLINE,itosh2((VBoutCount) / 16),WHITE);
	putstr(26,BORDERLINE,itos(per_byte,10,2),WHITE);
	putstr(29,BORDERLINE,itos(SmpcIntCount,10,1),WHITE);


	oregptr = 0;   			/*reset oregptr*/
	SmpcIntCount = 0;   /*reset Int count*/


}


/******************************************
 *                                        *
 * Vblank Out ISR
 *                                        *
 ******************************************/


void UserVblankEnd(void)
{
	VBoutCount++;
	SCL_VblankEnd();
}


/*----------------------------------------*
 *                                        *
 * Setup ISRs
 *                                        *
 -----------------------------------------*/


void SetVblank(void)
{
	SCL_VblankStart();

	INT_ChgMsk(INT_MSK_NULL,INT_MSK_VBLK_IN | INT_MSK_VBLK_OUT | INT_MSK_SYS);

	while( !((*((Uint32 *)0x25fe00a4)) & 2) );

	INT_SetScuFunc(INT_SCU_SYS,UserSmpcInt);
	INT_SetScuFunc(INT_SCU_VBLK_IN,UserVblankStart);
	INT_SetScuFunc(INT_SCU_VBLK_OUT,UserVblankEnd);
	INT_ChgMsk(INT_MSK_VBLK_IN | INT_MSK_VBLK_OUT | INT_MSK_SYS,INT_MSK_NULL);

}


/******************************************
 *                                        *
 * SMPC Interrupt Subroutine              *
 *                                        *
 ******************************************/

void UserSmpcInt(void)
{
	Uint8 i;


	set_imask(0xff);

	for(i = 0;i < 32; i++)					/* Copy OREGS into oreg[32*4 array] */
		oreg[oregptr+i] = (*((Uint8 *)(0x20100021 + i*2)));

	oregptr += 32;							    /* Pt to next set of OREGS */
	SmpcIntCount++;
	if(peekb(0x20100061) & 0x20){		/* SR Check Bit5 for remaining data */
		contflag ^= 0x80;             /* Toggle Continue Flag */
		pokeb(0x20100001,contflag);   /* IREG0 bit7 Continue Request */
	}
	set_imask(0);
}


/*----------------------------------------*
 *                                        *
 * REAL TIME CLOCK
 *                                        *
 -----------------------------------------*/


void rtc(void)
{

	putstr( 1,BORDERLINE-1,itosh2(oreg[1]),WHITE);
	putstr( 3,BORDERLINE-1,itosh2(oreg[2]),WHITE);
	putstr( 6,BORDERLINE-1,itos(oreg[3]&0x0f,10,2),WHITE);
	putstr( 9,BORDERLINE-1,itosh2(oreg[4]),WHITE);
	putstr(12,BORDERLINE-1,week[(oreg[3]>>4)&7],WHITE);
	putstr( 3,BORDERLINE,itosh2(oreg[5]),WHITE);
	putstr( 6,BORDERLINE,itosh2(oreg[6]),WHITE);
	putstr( 9,BORDERLINE,itosh2(oreg[7]),WHITE);

}


/******************************************
 *                                        *
 * GCLS
 *                                        *
 ******************************************/


void gcls(void)
{
	Uint32 *vram;
	Uint32 i;
	vram = (Uint32 *)SCL_VDP2_VRAM_B0;

	for(i = 0;i<0xa00;i++) *(vram++) = 0x40;
}


/******************************************
 *                                        *
 * STATUSLINE
 *                                        *
 ******************************************/


void statusline(void)
{
	box(0,BORDERLINE-2,38,2);
	putstr(5,BORDERLINE-1,"/  /  [   ] ML VI VO PD S",WHITE);
	putstr(5,BORDERLINE,":  :",WHITE);
	putstr(31,BORDERLINE,"@SEGA'95",BLUE);

}



/******************************************
 *                                        *
 * NOT AVAILABLE MESSAGE
 *                                        *
 ******************************************/


void NotAvailable(void)
{
	Uint32 i;
	Uint8 x,y;

	x=6,y=10;
	box(x,y,15,2);
	putstr(x+1,y+2,"NOT AVAILABLE!",WHITE);

	for(i=0;i<0x1fffff;i++);

	ProgramMode = MENU;
}


/*----------------------------------------*
 *                                        *
 * SET SCREEN MODE
 *                                        *
 -----------------------------------------*/


SetScreenMode()
{
	Uint32 i;
	Uint16 bg;

	SclVramConfig tp;
	SclConfig scfg,scfg1,scfg_nondisp;

	Uint16 cycle[] = {
		0xffff,0xffff,
		0x44ff,0xfeee,
		0xffff,0xffff,
		0xffff,0xffff
	};

	SCL_Vdp2Init();
	SCL_SetDisplayMode(SCL_NON_INTER,SCL_224LINE,SCL_NORMAL_A);

	SCL_InitVramConfigTb(&tp);

	tp.vramModeA = ON;
	tp.vramModeB = ON;

    tp.vramA0 = SCL_RBG0_CHAR;
    tp.vramA1 = SCL_NON;
    tp.vramB0 = SCL_RBG0_PN;
    tp.vramB1 = SCL_RBG0_K;

	SCL_SetVramConfig(&tp);

	SCL_InitRotateTable(SCL_VDP2_VRAM_B0+0x10000,1,SCL_RBG0,SCL_NON);


	SCL_InitConfigTb(&scfg);
	SCL_InitConfigTb(&scfg1);
	SCL_InitConfigTb(&scfg_nondisp);


	scfg.dispenbl = ON;
	scfg.charsize = SCL_CHAR_SIZE_1X1;
	scfg.platesize = SCL_PL_SIZE_1X1;
	scfg.pnamesize = SCL_PN2WORD;
	scfg.coltype  = SCL_COL_TYPE_256;
	scfg.datatype = SCL_CELL;
	scfg.mapover  = SCL_OVER_0;

	for(i = 0;i<16;i++)
		scfg.plate_addr[i] = SCL_VDP2_VRAM_B0;

	SCL_SetConfig(SCL_RBG0,&scfg);


	scfg1.dispenbl = ON;
	scfg1.coltype  = SCL_COL_TYPE_256;
	scfg1.datatype = SCL_BITMAP;
	scfg1.mapover  = SCL_OVER_0;
	scfg1.bmpsize  = SCL_BMP_SIZE_512X256;

	for(i = 0;i < 16;i++)
		scfg1.plate_addr[i] = SCL_VDP2_VRAM_A1;

	SCL_SetConfig(SCL_NBG0,&scfg1);

	scfg_nondisp.dispenbl = OFF;

	SCL_SetConfig(SCL_NBG1,&scfg_nondisp);
	SCL_SetConfig(SCL_NBG2,&scfg_nondisp);
	SCL_SetConfig(SCL_NBG3,&scfg_nondisp);

	SCL_SetPriority(SCL_RBG0,7);
	SCL_SetPriority(SCL_NBG0,5);
	SCL_SetPriority(SCL_NBG1 | SCL_NBG2 | SCL_NBG3 | SCL_RBG1,1);
    SCL_SetPriority(SCL_SP0|SCL_SP1|SCL_SP2|SCL_SP3|SCL_SP4|SCL_SP5|SCL_SP6,6);

	SCL_SetCycleTable(cycle);

	for(i=0;i<0x1c000;i+=4)
		pokel(0x25e20000+i,0x00000000);
	for(i=0;i<0x4000;i+=4)
		pokel(0x25e38000+i,0x03030303);

    bg = 2 + 2 * 32 + 5 * 1024;
	SCL_SetBack(SCL_VDP2_VRAM_B1+0x1f000,1,&bg);
}


/*********************** END OF SMPCDEM.C **********************************/
